-- ExtraTyreTracks
-- provides additional tyreTracks for dummy wheel nodes
--
-- @author  fruktor
-- @date    12/2014
-- @history v0.1 - initital implementation
--

ExtraTyreTracks = {};

function ExtraTyreTracks.prerequisitesPresent(specializations)
    return true;
end

function ExtraTyreTracks:load(savegame)
	self.extraTyreTracks = {};
    self.extraTyreTracks.tracks = {};
    self.extraTyreTracks.active = g_currentMission.tyreTrackSystem ~= nil;
    if self.extraTyreTracks.active then
        local i=0;
        while true do
            local base = string.format("vehicle.extraTyreTracks.track(%d)",i);
            if not hasXMLProperty(self.xmlFile,base) then
                break;
            end
            local track = {};
            local referenceWheelIndex = getXMLInt(self.xmlFile, string.format("%s#referenceWheelIndex", base));
            track.referenceWheel = self.wheels[referenceWheelIndex+1];
            track.node = Utils.indexToObject(self.components, getXMLString(self.xmlFile, string.format("%s#index", base)));
            track.radius = getXMLFloat(self.xmlFile, string.format("%s#radius", base));
            track.groundContactThreshold = Utils.getNoNil(getXMLFloat(self.xmlFile, string.format("%s#groundContactThreshold", base)), 0.05);
            local width = getXMLFloat(self.xmlFile, string.format("%s#width", base));
            local atlasIndex = Utils.getNoNil(getXMLInt(self.xmlFile, string.format("%s#atlasIndex", base)), 0);
            track.trackIndex = g_currentMission.tyreTrackSystem:createTrack(width, atlasIndex);
            table.insert(self.extraTyreTracks.tracks, track);
            i=i+1;
        end
    end
end

function ExtraTyreTracks:delete()
    if self.extraTyreTracks.active then
        for _,track in pairs(self.extraTyreTracks.tracks) do
            g_currentMission.tyreTrackSystem:destroyTrack(track.trackIndex);
        end
    end
end

function ExtraTyreTracks:readStream(streamId, connection)
    if self.extraTyreTracks.active and connection:getIsServer() then
        for _,track in pairs(self.extraTyreTracks.tracks) do
            if track.referenceWheel ~= nil then
                if track.referenceWheel.tyreTrackIndex == nil then
                    track.referenceWheel.contact = streamReadUIntN(streamId, 2);
                end;
            end;
        end;
    end;
end;

function ExtraTyreTracks:writeStream(streamId, connection)
    if self.extraTyreTracks.active and not connection:getIsServer() then
        for _,track in pairs(self.extraTyreTracks.tracks) do
            if track.referenceWheel ~= nil then
                if track.referenceWheel.tyreTrackIndex == nil then
                    streamWriteUIntN(streamId, track.referenceWheel.contact, 2);
                end;
            end;
        end;
    end;
end

function ExtraTyreTracks:readUpdateStream(streamId, timestamp, connection)
    if self.extraTyreTracks.active and connection:getIsServer() then
        for _,track in pairs(self.extraTyreTracks.tracks) do
            if track.referenceWheel ~= nil then
                if track.referenceWheel.tyreTrackIndex == nil then
                    track.referenceWheel.contact = streamReadUIntN(streamId, 2);
                end;
            end;
        end;
    end;
end

function ExtraTyreTracks:writeUpdateStream(streamId, connection, dirtyMask)
    if self.extraTyreTracks.active and not connection:getIsServer() then
        for _,track in pairs(self.extraTyreTracks.tracks) do
            if track.referenceWheel ~= nil then
                if track.referenceWheel.tyreTrackIndex == nil then
                    streamWriteUIntN(streamId, track.referenceWheel.contact, 2);
                end;
            end;
        end;
    end;
end

function ExtraTyreTracks:mouseEvent(posX, posY, isDown, isUp, button)
end

function ExtraTyreTracks:keyEvent(unicode, sym, modifier, isDown)
end

function ExtraTyreTracks:update(dt)
    if self.extraTyreTracks.active then
        for _,track in pairs(self.extraTyreTracks.tracks) do
            if track.referenceWheel ~= nil then
                local x,y,z = getWorldTranslation(track.node);
                local th = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x,0,z);
                if math.abs(y - track.radius - th) < track.groundContactThreshold then
                    local color = track.referenceWheel.lastColor;
                    if color ~= nil then
                        local ux,uy,uz = localDirectionToWorld(self.rootNode, 0,1,0);
                        g_currentMission.tyreTrackSystem:addTrackPoint(track.trackIndex, x,th,z, ux,uy,uz, color[1], color[2], color[3], track.referenceWheel.dirtAmount, color[4], 1);
                    end
                else
                    g_currentMission.tyreTrackSystem:cutTrack(track.trackIndex);
                end
            end
        end
    end
end

function ExtraTyreTracks:updateTick(dt)
end

function ExtraTyreTracks:draw()
end

-- converted